#include <plib.h>
#include <p32xxxx.h>
#include "./App1632_LCM.h"

/* Configuration Settings */
// OSC Input = HS w/PLL
// CPU Clock = ( ( OSC Input / 2 ) * 15 ) / 1
// PB Clock = CPU Clock / 1
// Disabld WDT , ICSP/ICD = PGC4 , PGD4
#pragma config FNOSC = PRIPLL , POSCMOD = HS
#pragma config FPLLIDIV = DIV_2 , FPLLMUL = MUL_15 , FPLLODIV = DIV_1
#pragma config FPBDIV = DIV_8
#pragma config FWDTEN = OFF , ICESEL = ICS_PGx4

#define SystemFrequency		60000000L
#define PBFrequency		SystemFrequency / 8
#define Timer1Tick		( PBFrequency / 256 / TogglesPerSec )
#define TogglesPerSec		10

const unsigned char LCMString1[ ] = "PIC32MX250F128D ";
const unsigned char LCMString2[ ] = "VR1:";
unsigned int VR1Value = 0;

void __ISR(_TIMER_1_VECTOR, ipl4) ISR_Timer1(void)
{
    // 清除Timer中斷旗標。
    mT1ClearIntFlag();

    // 使用C32 ADC Function啟動ADC的轉換。
    ConvertADC10();

    // LED狀態反向。
    mPORTAToggleBits(BIT_9);
    mPORTCToggleBits(BIT_3);
}

int main(void)
{
    // disable JTAG port
    DDPCONbits.JTAGEN = 0;

    // 周邊時脈 = 系統時脈 / 8。
    OSCSetPBDIV(OSC_PB_DIV_8);

    // 使用C32的SYSTEM Config巨集將CPU的操作最佳化，但不修改到FPBDIV的設定。
    SYSTEMConfig(SystemFrequency, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);
    // SYSTEMConfigPerformance( SystemFrequency  );
    // SYSTEMConfigWaitStates( SystemFrequency );

    // LED對應之I/O Port設定成數位輸出，並指定I/O Port狀態。
    mPORTASetPinsDigitalOut(BIT_9);
    mPORTCSetPinsDigitalOut(BIT_3);
    mPORTAClearBits(BIT_9);
    mPORTCSetBits(BIT_3);

    // 啟用Timer，並指定其中斷優先權。
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_4);

    // Timer設定。
    OpenTimer1(T1_ON | T1_IDLE_CON | T1_GATE_OFF | T1_PS_1_256 | T1_SOURCE_INT, Timer1Tick);

    // LCM初始化。
    LCM_Init();

    // 設定遊標，輸出字串。
    LCM_SetCursor(0, 0);
    LCM_PutROMString(LCMString1);

    // 使用C32 ADC10 Function設定ADC。
    OpenADC10(ADC_MODULE_ON | ADC_IDLE_CONTINUE | ADC_FORMAT_INTG16 | ADC_CLK_MANUAL | ADC_AUTO_SAMPLING_ON | ADC_SAMP_OFF,
            ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF,
            ADC_SAMPLE_TIME_31 | ADC_CONV_CLK_SYSTEM | ADC_CONV_CLK_32Tcy,
            ENABLE_AN6_ANA, SKIP_SCAN_ALL);

    // 使用C32 ADC10 Function設定AD正確的正,負通道。
    SetChanADC10(ADC_CH0_POS_SAMPLEA_AN6 | ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEB_AN0 | ADC_CH0_NEG_SAMPLEB_NVREF);

    //    // 使用C32 I/O Port Function將正確AD通道設定為類比輸入。
    //  PORTSetPinsAnalogIn(IOPORT_B, BIT_2);

    INTEnableSystemMultiVectoredInt();

    while (1)
    {
        // 轉換完成 ?
        if (INTGetFlag(INT_AD1))
        {
             // 讀取ADC轉換之數值。
            VR1Value = (unsigned int) ReadADC10(0);

            // 清除ADC中斷旗標
            INTClearFlag(INT_AD1);

			// 設定遊標，輸出ADC數值。
            LCM_SetCursor(0, 1);
            LCM_PutROMString(LCMString2);
            LCM_PutNumber(VR1Value, 4);
        }
    }

    while (1);
}
