#include <plib.h>
#include <p32xxxx.h>
#include ".\App1632_LCM.h"

/* Configuration Settings */
// OSC Input = HS w/PLL
// CPU Clock = ( ( OSC Input / 2 ) * 15 ) / 1
// PB Clock = CPU Clock / 1
// Disabld WDT , ICSP/ICD = PGC4 , PGD4
#pragma config FNOSC = PRIPLL , POSCMOD = HS
#pragma config FPLLIDIV = DIV_2 , FPLLMUL = MUL_15 , FPLLODIV = DIV_1
#pragma config FPBDIV = DIV_8
#pragma config FWDTEN = OFF , ICESEL = ICS_PGx4

#define SystemFrequency		60000000L
#define PBFrequency		SystemFrequency / 8
#define Timer1Tick		( PBFrequency / 256 / TogglesPerSec )
#define TogglesPerSec		10

const unsigned char LCMString1[ ] = "PIC32MX250F128D ";
const unsigned char LCMString2[ ] = "Write:  ,Read:  ";
unsigned char Timer1TickFlag = 0;

void I2C_EE16BitsByteWirte(unsigned char SalveAddr, unsigned int Addr, unsigned char Data);
unsigned char I2C_EE16BitsRandomRead(unsigned char SalveAddr, unsigned int Addr);

#define I2CChannel I2C2

void __ISR(_TIMER_1_VECTOR, ipl4) ISR_Timer1(void)
{
    Timer1TickFlag++;

    // 清除Timer中斷旗標。
    mT1ClearIntFlag();

    // LED狀態反向。
    mPORTAToggleBits(BIT_9);
    mPORTCToggleBits(BIT_3);
}

int main(void)
{
    unsigned long i = 0;
    unsigned int I2CEEAddr = 0;

    // disable JTAG port
    DDPCONbits.JTAGEN = 0;

    // 周邊時脈 = 系統時脈 / 8。
    OSCSetPBDIV(OSC_PB_DIV_8);

    // 使用C32的SYSTEM Config巨集將CPU的操作最佳化，但不修改到FPBDIV的設定。
    SYSTEMConfig(SystemFrequency, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);
    // SYSTEMConfigPerformance( SystemFrequency  );
    // SYSTEMConfigWaitStates( SystemFrequency );

    // LED對應之I/O Port設定成數位輸出，並指定I/O Port狀態。
    mPORTASetPinsDigitalOut(BIT_9);
    mPORTCSetPinsDigitalOut(BIT_3);
    mPORTAClearBits(BIT_9);
    mPORTCSetBits(BIT_3);

    // 啟用Timer，並指定其中斷優先權。
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_4);

    // Timer設定。
    OpenTimer1(T1_ON | T1_IDLE_CON | T1_GATE_OFF | T1_PS_1_256 | T1_SOURCE_INT, Timer1Tick);

    // LCM初始化。
    LCM_Init();

    // 設定遊標，輸出字串。
    LCM_SetCursor(0, 0);
    LCM_PutROMString(LCMString1);
    LCM_SetCursor(0, 1);
    LCM_PutROMString(LCMString2);

    // Disable Analog Function
    mPORTBSetPinsDigitalOut(BIT_2 | BIT_3);

    // 使用XC32 I2C Function初始化I2C Module。
    I2CConfigure(I2CChannel, 0);
    I2CSetFrequency(I2CChannel, PBFrequency, 100000);
    I2CSetSlaveAddress(I2CChannel, 0x00, 0x00, I2C_USE_7BIT_ADDRESS);
    I2CEnable(I2CChannel, 1);

    INTEnableSystemMultiVectoredInt();

    while (1)
    {
        if (Timer1TickFlag >= 1)
        {
            Timer1TickFlag--;

            LCM_SetCursor(6, 1);
            LCM_PutHex(I2CEEAddr);
            I2C_EE16BitsByteWirte(0xA0, I2CEEAddr, I2CEEAddr);

            // ! EEPROM的寫入, 約需要5mS, 期間如果進行存取, 會中斷寫入工作,
            // 此處以簡單的軟體延遲處理, 但由於不同版本的編譯器,
            // 對於軟體延遲所產生的時間間隔不一, 必須特別注意!
            // 或者改採用硬體的計時來處理.
            for (i = 0; i < 50000L; i++);

            LCM_SetCursor(14, 1);
            LCM_PutHex(I2C_EE16BitsRandomRead(0xA0, I2CEEAddr));

            if (++I2CEEAddr > 0xfff)
                I2CEEAddr = 0;
        }
    }

    while (1);
}

void I2C_EE16BitsByteWirte(unsigned char SalveAddr, unsigned int Addr, unsigned char Data)
{
    while (!I2CBusIsIdle(I2CChannel));

    I2CStart(I2CChannel);
    while (!(I2CGetStatus(I2CChannel) & I2C_START));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, SalveAddr & 0xfffe);
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, (unsigned char) ((Addr >> 8) & 0x00ff));
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, (unsigned char) (Addr & 0x00ff));
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, Data);
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    I2CStop(I2CChannel);
}

unsigned char I2C_EE16BitsRandomRead(unsigned char SalveAddr, unsigned int Addr)
{
    unsigned char Data = 0;

    while (!I2CBusIsIdle(I2CChannel));

    I2CStart(I2CChannel);
    while (!(I2CGetStatus(I2CChannel) & I2C_START));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, SalveAddr & 0xfffe);
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, (unsigned char) ((Addr >> 8) & 0x00ff));
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, (unsigned char) (Addr & 0x00ff));
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    I2CRepeatStart(I2CChannel);
    while (!(I2CGetStatus(I2CChannel) & I2C_START));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, SalveAddr | 0x0001);
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    I2CReceiverEnable(I2CChannel, 1);
    while (!I2CReceivedDataIsAvailable(I2CChannel));
    Data = I2CGetByte(I2CChannel);
 
    I2CAcknowledgeByte(I2CChannel, FALSE);
    while (!I2CAcknowledgeHasCompleted(I2CChannel));

    I2CStop(I2CChannel);

    return Data;
}
