#include <plib.h>
#include <p32xxxx.h>
#include ".\App1632_LCM.h"

/* Configuration Settings */
// OSC Input = HS w/PLL
// CPU Clock = ( ( OSC Input / 2 ) * 15 ) / 1
// PB Clock = CPU Clock / 1
// Disabld WDT , ICSP/ICD = PGC4 , PGD4
// Secondary Oscillator Disabled
#pragma config FNOSC = PRIPLL , POSCMOD = HS
#pragma config FPLLIDIV = DIV_2 , FPLLMUL = MUL_15 , FPLLODIV = DIV_1
#pragma config FPBDIV = DIV_8
#pragma config FWDTEN = OFF , ICESEL = ICS_PGx4
#pragma config FSOSCEN = OFF

#define SystemFrequency		60000000L
#define PBFrequency		SystemFrequency / 8
#define Timer1Tick		( PBFrequency / 256 / TogglesPerSec )
#define TogglesPerSec		10

const unsigned char LCMString1[ ] = "PIC32MX250F128D ";
const unsigned char LCMString2[ ] = "Write:  ,Read:  ";
const unsigned char LCMString3[ ] = "Flash Erasing ! ";
unsigned char Timer1TickFlag = 0;

#define EE_CS LATBbits.LATB4
#define EE_CS_TRIS TRISBbits.TRISB4

void SPI_EEByteWirte(unsigned int Addr, unsigned char Data);
unsigned char SPI_EEByteRead(unsigned int Addr);
void SPI_EEEnableWrite(void);
void SPI_FlashDisableProtect(void);
void SPI_FlashChipErase(void);

#define SPIChannel SPI_CHANNEL1

void __ISR(_TIMER_1_VECTOR, ipl4) ISR_Timer1(void)
{
    Timer1TickFlag++;

    // 清除Timer中斷旗標。
    mT1ClearIntFlag();

    // LED狀態反向。
    mPORTAToggleBits(BIT_9);
    mPORTCToggleBits(BIT_3);
}

int main(void)
{
    unsigned long i = 0;
    unsigned int SPIEEAddr = 0;

    // disable JTAG port
    DDPCONbits.JTAGEN = 0;

    // 周邊時脈 = 系統時脈 / 8。
    OSCSetPBDIV(OSC_PB_DIV_8);

    // 使用C32的SYSTEM Config巨集將CPU的操作最佳化，但不修改到FPBDIV的設定。
    SYSTEMConfig(SystemFrequency, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);
    // SYSTEMConfigPerformance( SystemFrequency  );
    // SYSTEMConfigWaitStates( SystemFrequency );

    // LED對應之I/O Port設定成數位輸出，並指定I/O Port狀態。
    mPORTASetPinsDigitalOut(BIT_9);
    mPORTCSetPinsDigitalOut(BIT_3);
    mPORTAClearBits(BIT_9);
    mPORTCSetBits(BIT_3);

    // 啟用Timer，並指定其中斷優先權。
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_4);

    // Timer設定。
    OpenTimer1(T1_ON | T1_IDLE_CON | T1_GATE_OFF | T1_PS_1_256 | T1_SOURCE_INT, Timer1Tick);

    // LCM初始化。
    LCM_Init();

    // 設定遊標，輸出字串。
    LCM_SetCursor(0, 0);
    LCM_PutROMString(LCMString1);

    // 使用C32 SPI Function初始化SPI Module。
    // SPI Chip Select
    mPORTBSetPinsDigitalOut(BIT_4);
    mPORTBSetBits(BIT_4);

    // 使用XC32 PPS Function指定SPI SDO, SDI的腳位。
    // 注意 : 設定前記得先解鎖PPS, 設定為後再上鎖。
    PPSUnLock;
    PPSOutput(3, RPA4, SDO1); //Assign SDO1 to pin RPA4
    PPSInput(2, SDI1, RPA8); //Assign U2RX to pin RPA8
    PPSLock;

    SpiChnConfigure(SPIChannel, SPI_OPEN_MSTEN | SPI_OPEN_CKE_REV | SPI_OPEN_MODE8 | SPI_OPEN_SIDL);
    SpiChnSetBitRate(SPIChannel, PBFrequency, 100000);
    SpiChnEnable(SPIChannel, 1);

    // Disable All Blocks Protection
    SPI_EEEnableWrite();
    SPI_FlashDisableProtect();

    //Chip erase
    LCM_SetCursor(0, 1);
    LCM_PutROMString(LCMString3);
    SPI_EEEnableWrite();
    SPI_FlashChipErase();
    for (i = 0; i < 1000000L; i++) //wait for Chip erase

        INTEnableSystemMultiVectoredInt();

    // 設定遊標，輸出字串。
    LCM_SetCursor(0, 1);
    LCM_PutROMString(LCMString2);

    while (1)
    {
        if (Timer1TickFlag >= 1)
        {
            Timer1TickFlag--;

            SPI_EEEnableWrite();

            LCM_SetCursor(6, 1);
            LCM_PutHex(SPIEEAddr);
            SPI_EEByteWirte(SPIEEAddr, SPIEEAddr);

            // 此處以簡單的軟體延遲處理, 但由於不同版本的編譯器,
            // 對於軟體延遲所產生的時間間隔不一, 必須特別注意!
            // 或者改採用硬體的計時來處理.
            for (i = 0; i < 200; i++); //wait 7us for each byte internal cell program time

            LCM_SetCursor(14, 1);
            LCM_PutHex(SPI_EEByteRead(SPIEEAddr));

            if (++SPIEEAddr > 0xfffff)
                SPIEEAddr = 0;
        }
    }
    while (1);
}

void SPI_EEByteWirte(unsigned int Addr, unsigned char Data)
{
    unsigned long i = 0;

    // CS Active
    EE_CS = 0;
    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, 0x02);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, (Addr >> 16) & 0x00ff);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, (Addr >> 8) & 0x00ff);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, Addr & 0xff);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, Data);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    // Wait For Last Byte Transmitted
    for (i = 0; i < 10; i++);

    // CS Inactive
    EE_CS = 1;
    for (i = 0; i < 10; i++);
}

unsigned char SPI_EEByteRead(unsigned int Addr)
{
    unsigned char Data = 0;
    unsigned long i = 0;

    // CS Active
    EE_CS = 0;

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, 0x03);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, (Addr >> 16) & 0x00ff);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, (Addr >> 8) & 0x00ff);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, Addr & 0xff);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, Data);
    while (!SpiChnDataRdy(SPIChannel));
    Data = SpiChnReadC(SPIChannel);

    // Wait For Last Byte Transmitted
    for (i = 0; i < 10; i++);

    // CS Inactive
    EE_CS = 1;
    for (i = 0; i < 10; i++);

    return Data;
}

void SPI_EEEnableWrite(void)
{
    unsigned long i = 0;

    // CS Active
    EE_CS = 0;

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, 0x06);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    // Wait For Last Byte Transmitted
    for (i = 0; i < 10; i++);

    // CS Inactive
    EE_CS = 1;
    for (i = 0; i < 10; i++);
}

void SPI_FlashChipErase(void)
{
    unsigned long i = 0;

    // CS Active
    EE_CS = 0;

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, 0x60);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    // Wait For Last Byte Transmitted
    for (i = 0; i < 10; i++);

    // CS Inactive
    EE_CS = 1;
    for (i = 0; i < 10; i++);
}

void SPI_FlashDisableProtect(void)
{
    unsigned long i = 0;

    // CS Active
    EE_CS = 0;

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, 0x01);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, 0x00);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    // Wait For Last Byte Transmitted
    for (i = 0; i < 10; i++);

    // CS Inactive
    EE_CS = 1;
    for (i = 0; i < 10; i++);
}
